package com.ibm.mqe.bridgemodel;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.mqe.mqbridge.*;
import com.ibm.uia.metamodel.*;
import com.ibm.uia.renderingInterface.*;
import com.ibm.uia.xml.*;
import com.ibm.mqe.model.*;
import com.ibm.mqe.*;
import java.util.*;
/**
 * Insert the type's description here.
 * Creation date: (12/07/2000 13:05:30)
 * @author: Phill van Leersum
 */
public abstract class AbstractBridgeObject extends CreatableObject  implements BridgeModelConstants {
	protected static final String CMD_START = "Start";
	protected static final String CMD_STARTALL = "Start All";
	protected static final String CMD_STOP = "Stop";
	protected static final String CMD_DELETE_ALL = "Delete All";
	private boolean running = false;
/**
 * Delete the resource assciated with the receiver.
 * If no listener has been set then the method waits for a response to 
 * the admin message, and returns it.
 * If a listener has been set, then the message will be returned
 * asynchronously via the listeners acceptMessage() method.
 */
public MQeAdminMsg _delete() {
	MQeAdminMsg replyMsg = null;
	try {
		MQeAdminMsg msg = adminMessage();
		msg.delete(new MQeFields());
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		_warning("exception deleting bridge object", e);
	}
	return replyMsg;
}
/**
 * Delete the resource assciated with the receiver.
 * If no listener has been set then the method waits for a response to 
 * the admin message, and returns it.
 * If a listener has been set, then the message will be returned
 * asynchronously via the listeners acceptMessage() method.
 */
public MQeAdminMsg _deleteAll() {
	MQeAdminMsg replyMsg = null;
	try {
		MQeMQBridgesAdminMsg msg = bridgeTypeAdminMessage();
		msg.delete(new MQeFields());
		msg.getInputFields().putBoolean( 
					MQeCharacteristicLabels.MQE_FIELD_LABEL_AFFECT_CHILDREN, true);
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		_warning("exception deleting bridge object", e);
	}
	return replyMsg;
}
/**
 * _readRefreshData method comment.
 * Because each MQBridge type object has a single child type, we
 * can generalise this method, with only the child type name looked
 * up polymorphically.
 */
protected boolean _readRefreshData(com.ibm.mqe.MQeFields data, java.util.Vector found) {
	boolean reply = super._readRefreshData(data, found);
	try {
		int i = data.getInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_RUN_STATE);
		running = (MQeRunState.RUN_STATE_RUNNING == i);
		
		MQeFields[] allChildData = data.getFieldsArray(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILDREN);
		for (i = 0; i < allChildData.length; i++){
			String childType = getChildTypeName();
			String childName = allChildData[i].getUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILD);
			UIObject uio = findExistingChild(childType, childName);
			if (null == uio) {
				uio = addNewChild(childType, childName);
			}
			if (null != uio) {
				((CreatableObject)uio).setStateCreated();
				((MQeObject)uio).refresh();
				found.addElement(uio);
			}
		}
	} catch (Exception e) {
	}
	return reply;
}
public MQeAdminMsg _start(boolean affectChildren) {
	MQeAdminMsg reply = null;
	try {
		MQeMQBridgesAdminMsg msg = bridgeTypeAdminMessage();
		msg.setName(getName());
		msg.start(affectChildren);
		reply = putAdminMessage(msg);
	} catch (Exception e) {
		_warning("exception while starting bridge thing", e);
	}
	return reply;
}
public MQeAdminMsg _stop() {
	MQeAdminMsg reply = null;
	try {
		MQeMQBridgesAdminMsg msg = bridgeTypeAdminMessage();
		msg.setName(getName());
		msg.stop();
		reply = putAdminMessage(msg);
	} catch (Exception e) {
		_warning("exception while stopping bridge thing", e);
	}
	return reply;
}
/**
 * reply an admin message for the MQe resource 
 * represented by the receiver.
 */
protected MQeAdminMsg adminMessage() {
	MQeAdminMsg msg = bridgeTypeAdminMessage();
	return msg;
}
protected abstract MQeMQBridgesAdminMsg bridgeTypeAdminMessage();
public String getCaptionText() {
	String str = super.getCaptionText();
	if (running) {
		str = str + "(running)";
	} else {
		str = str + "(stopped)";
	}
	return str;
}
protected abstract String getChildTypeName();
/**
 * This is the method called to handle all reply messages, whether delivered
 * asynchronously (through the callback method 'acceptMessage') or synchronously
 * as a returned value from a stub operation.
 */
protected boolean handleMessage(MQeAdminMsg msg) {
	boolean changed = false;
	if (null != msg) {
		if (msg instanceof MQeMQBridgesAdminMsg) {
			MQeFields data = msg.getOutputFields();
			switch (_getAction(msg)) {
				case MQeMQBridgesAdminMsg.Action_Start :
					//if (_isSuccess(msg)) {
						//started(false);
						refresh();
					//}
					break;
				case MQeMQBridgesAdminMsg.Action_Stop :
					//if (_isSuccess(msg)) {
						//stopped();
						refresh();
					//}
					break;
				default :
					changed = super.handleMessage(msg);
					break;
			}
		} else {
			changed = super.handleMessage(msg);
		}
	}
	return changed;
}
/**
 * Reply true if the supplied renderable operation is currently enabled 
 * for the receiver.  Reply false if the operation is not currently
 * enabled.
 */
public boolean isOperationEnabled(RenderableOperation rop) {
	boolean reply = true;
	String cmd = rop.getName();
	if (CMD_DELETE.equals(cmd)) {
		if (getAllChildren().hasMoreElements()) {
			reply = false;
		}
	} else if (CMD_DELETE_ALL.equals(cmd)) {
		reply = getAllChildren().hasMoreElements();
	} else {
		reply = super.isOperationEnabled(rop);
	}
	return reply;
}
protected boolean performCreate() {
	boolean reply = super.performCreate();
	MQeAdminMsg replyMsg = null;
	try {
		MQeAdminMsg msg = adminMessage();
		MQeFields initialValues = new MQeFields();
		setInitialValues(initialValues);
		msg.create(initialValues);
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		_warning("exception creating bridge object", e);
	}
	reply = handleMessage(replyMsg);
	return reply; 
}
protected boolean performDelete() {
	boolean reply = super.performDelete();
	MQeAdminMsg msg = null;
	if (getAllChildren().hasMoreElements()) {
		msg = _deleteAll();
	} else {
		msg = _delete();
	}
	reply = handleMessage(msg);
	return reply; 
}
/**
 * Perform the actions represented by the supplied instance of 
 * <code>RenderableOperation</code> defined in te metamodel file.
 * If the operation was defined with a prompt then the String array
 * will be non-null, and will contain a string for every line of the prompt
 *
 * The renderer responsible for delivering the operation request is passed in
 * for convenience.
 *
 * Reply true if a stuctural refresh is reqired.
 */
public boolean performOperation(RenderableOperation op, Renderer renderer) {
	String[] data = op.getData();
	String command = op.getName();
	boolean reply = false;
	if (CMD_START.equals(command)) {
		_start(false);
		reply = true;
	} else if (CMD_STARTALL.equals(command)) {
		_start(true);
		reply = true;
	} else if (CMD_STOP.equals(command)) {
		_stop();
		reply = true;
	} else if (CMD_DELETE_ALL.equals(command)) {
		reply = delete();
	} else {
		reply = super.performOperation(op, renderer);
	}
	return reply;
}
public boolean refresh() {
	boolean reply = false;
	if (isConnected()) {
		MQeAdminMsg replyMsg = null;
		try {
			MQeAdminMsg msg = adminMessage();
			msg.setAction(MQeAdminMsg.Action_InquireAll);
			replyMsg = putAdminMessage(msg);
		} catch (Exception e) {
			e.printStackTrace();
		}
		reply = handleMessage(replyMsg);
	}
	return reply;
}
protected void setInitialValues(MQeFields mqef) {
}
protected void started(boolean allChildren) {
	running = true;
	if (allChildren) {
		Enumeration enum = getAllChildren();
		while (enum.hasMoreElements()) {
			((AbstractBridgeObject)enum).started(allChildren);
		}
	}
}
protected void stopped() {
	running = false;
	Enumeration enum = getAllChildren();
	while (enum.hasMoreElements()) {
		((AbstractBridgeObject)enum).stopped();
	}
}
}
