package com.ibm.mqe.bridgemodel;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.mqe.*;
import com.ibm.mqe.mqbridge.*;
import com.ibm.uia.metamodel.*;
import com.ibm.uia.renderingInterface.*;
import com.ibm.uia.xml.*;
import com.ibm.mqe.model.*;
/**
 * Insert the type's description here.
 * Creation date: (12/07/2000 13:05:30)
 * @author: Phill van Leersum
 */
public class MQeBridgeQMProxy extends AbstractBridgeObject {
	private static final String CMD_CREATE_CONNECTION = "Connection";
/**
 * _readRefreshData method comment.
 * Because each MQBridge type object has a single child type, we
 * can generalise this method, with only the child type name looked
 * up polymorphically.
 */
protected boolean _readRefreshData(com.ibm.mqe.MQeFields data, java.util.Vector found) {
	boolean reply = super._readRefreshData(data, found);
	try {
		
		com.ibm.mqe.MQeFields[] allChildData = data.getFieldsArray(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILDREN);
		for (int i = 0; i < allChildData.length; i++){
			String childType = getChildTypeName();
			String childName = allChildData[i].getUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_CHILD);
			UIObject uio = findExistingChild(childType, childName);
			if (null == uio) {
				uio = addNewChild(childType, childName);
			}
			if (null != uio) {
				((CreatableObject)uio).setStateCreated();
				((MQeObject)uio).refresh();
				found.addElement(uio);
			}
		}
	} catch (Exception e) {
		_comment("error reading refresh data", e);
	}
	return reply;
}
/**
 * adminMessage method comment.
 */
protected MQeMQBridgesAdminMsg bridgeTypeAdminMessage() {
	return containedQMAdminMessage();
}
/**
 * adminMessage method comment.
 */
private MQeMQQMgrProxyAdminMsg containedQMAdminMessage() {
	MQeMQQMgrProxyAdminMsg msg = null;
	try {
		msg = new MQeMQQMgrProxyAdminMsg();
		msg.putMQQMgrProxyName(getName());
		msg.putBridgeName(getBridge().getName());
	} catch (Exception e) {
		_error("error creating MQeMQQMProxyAdminMsg", e);;
	}
	return msg;
}
private boolean createConnection(Renderer renderer, String[] data) {
	boolean changed = false;
	if (null != data) {
		String bridgeName = data[0];
		String type = BRIDGE_CONN_TYPE_NAME;
		UIObject uio = findExistingChild(type, bridgeName);
		if (null == uio) {
			uio = addNewChild(type, bridgeName);
		}
		if (null != uio) {
			uio.setAttribute(CONNECTION_PORT_ATTRIBUTE, data[1]);
			((MQeObject)uio).commit(false);
		}
		changed = true;
	}
	return changed;
}
public final MQeBridgeProxy getBridge() {
	return ((MQeBridgeProxy)getParent());
}
protected String getChildTypeName() {
	return BRIDGE_CONN_TYPE_NAME;
}
/**
 * Perform the actions represented by the supplied instance of 
 * <code>RenderableOperation</code> defined in te metamodel file.
 * If the operation was defined with a prompt then the String array
 * will be non-null, and will contain a string for every line of the prompt
 *
 * The renderer responsible for delivering the operation request is passed in
 * for convenience.
 *
 * Reply true if a stuctural refresh is required.
 */
public boolean performOperation(RenderableOperation op, Renderer renderer) {
	String[] data = op.getData();
	String command = op.getName();
	boolean reply = false;
	if (CMD_CREATE_CONNECTION.equals(command)) {
		reply = createConnection(renderer, data);
	} else {
		reply = super.performOperation(op, renderer);
	}
	return reply;
}
protected void setInitialValues(MQeFields mqef) {
	try {
		String hostname = getAttributeValue(QMHOSTNAME_ATTRIBUTE);
		mqef.putUnicode( MQeCharacteristicLabels.MQE_FIELD_LABEL_HOST_NAME, hostname);
	} catch (Exception e) {
	}
}
}
