package com.ibm.mqe.model;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.renderingInterface.*;
import com.ibm.uia.metamodel.*;
import com.ibm.mqe.administration.*;
import com.ibm.mqe.*;
import java.util.*;

/**
 * Insert the type's description here.
 * Creation date: (08/06/2000 14:49:45)
 * @author: Phill van Leersum
 */
public final class StoreAndForwardQueue extends AbstractRemoteQueue {
	private static final String CMD_ADD_QM = "Add QueueManager";
private final boolean _checkAddQmReply(MQeStoreAndForwardQueueAdminMsg replyMsg) {
	boolean reply = false;
	try {
		if (replyMsg == null) {
			reply = false;
		} else if (_isSuccess(replyMsg)) {
			MQeFields mqef = replyMsg.getOutputFields();
			String[] qms = mqef.getAsciiArray(replyMsg.Queue_QMgrNameList);
			for (int i = 0; i < qms.length; i++){
				CreatableObject mqeo = (CreatableObject)findExistingChild(QUEUEMANAGER_ENTRY_TYPE_NAME, qms[i]);
				if (null == mqeo) {
					mqeo = (CreatableObject)addNewChild(QUEUEMANAGER_ENTRY_TYPE_NAME, qms[i]);
				}
				if (null != mqeo) {
					mqeo.setStateCreated();
				}
			}
			reply = true;
		} else {
			String fieldName = replyMsg.Queue_QMgrNameList;
			MQeFields input = replyMsg.getInputFields();
			String qmeName = input.getAsciiArray(fieldName)[0];
			CreatableObject qme = (CreatableObject) findExistingChild(QUEUEMANAGER_ENTRY_TYPE_NAME, qmeName);
			if (null != qme) {
				qme.setStateToBeCreated();
			}
			String reason;
			if (replyMsg.getRC() == MQeAdminMsg.RC_Mixed) {
				MQeFields errors = replyMsg.getErrorFields();
				reason = errors.getAsciiArray(fieldName)[0];
			} else {
				reason = replyMsg.getReason();
			}
			_warning("failed to create qmEntry: " + qmeName + " because: " + reason);
			reply = true;
		}
		/*if (replyMsg.getRC() == MQeAdminMsg.RC_Mixed) {
			reply = false;
			MQeFields errors = replyMsg.getErrorFields();
			Enumeration en = errors.fields();
			while (reply && en.hasMoreElements()) {
				String value[];
				String name = (String) en.nextElement();
				value = replyMsg.getFieldInError(name);
				for (int j = 0; j < value.length; j++) {
					_warning("Field in error: " + name + "  " + value[j]);
				}
			}
		} else {
			_warning("..Action Failed: " + replyMsg.getReason());
		}*/
	} catch (Exception e) {
		_error("add qm reply", e);
	}
	return reply;
}
private final boolean _checkRemoveQmReply(MQeStoreAndForwardQueueAdminMsg replyMsg) {
	boolean reply = false;
	try {
		if (replyMsg == null) {
			reply = false;
		} else if (replyMsg.getRC() == MQeAdminMsg.RC_Success) {
			MQeFields mqef = replyMsg.getOutputFields();
			String[] qms = mqef.getAsciiArray(replyMsg.Queue_QMgrNameList);
			for (int i = 0; i < qms.length; i++){
				UIObject uio = findExistingChild(QUEUEMANAGER_ENTRY_TYPE_NAME, qms[i]);
				if (null != uio) {
					removeChild(uio);
				}
			}
			reply = true;
		} else {
			String fieldName = replyMsg.Queue_QMgrNameList;
			MQeFields input = replyMsg.getInputFields();
			String qmeName = input.getAsciiArray(fieldName)[0];
			CreatableObject qme = (CreatableObject) findExistingChild(QUEUEMANAGER_ENTRY_TYPE_NAME, qmeName);
			if (null != qme) {
				qme.setStateToBeDeleted();
			}
			String reason;
			if (replyMsg.getRC() == MQeAdminMsg.RC_Mixed) {
				MQeFields errors = replyMsg.getErrorFields();
				reason = errors.getAsciiArray(fieldName)[0];
			} else {
				reason = replyMsg.getReason();
			}
			_warning("failed to delete qmEntry: " + qmeName + " because: " + reason);
			reply = true;
		}
	} catch (Exception e) {
		reply = false;
	}
	return reply;
}
/**
 * Read the receivers data from the output data field
 */
boolean _readQmData(MQeFields data, Vector found) {
	boolean changed = false;
	String[] alii = null;
	try {
		alii = data.getAsciiArray(MQeStoreAndForwardQueueAdminMsg.Queue_QMgrNameList);
	} catch (Exception e) {
		//_log().println("No queue managers found");
		//e.printStackTrace(_log());
	}
	if (null != alii) {
		for (int i = 0; i < alii.length; i++){
			CreatableObject uio = (CreatableObject)findExistingChild(QUEUEMANAGER_ENTRY_TYPE_NAME, alii[i]);
			if (null == uio) {
				uio = (CreatableObject)addNewChild(QUEUEMANAGER_ENTRY_TYPE_NAME, alii[i]);
				changed = true;
			}
			if (null != uio) {
				found.addElement(uio);
				uio.setStateCreated();
			}
		}
	}
	return changed;
}
/**
 * Read the receivers data from the output data field
 */
protected boolean _readRefreshData(MQeFields data, Vector found) {
	boolean reply = super._readRefreshData(data, found);
	if (_readQmData(data, found) || reply) {
		reply = true;
	}
	return reply;
}
final boolean _reallyAddQueueManager(String newQmName) {
	boolean reply = false;
	if (null != newQmName) {
		MQeStoreAndForwardQueueAdminMsg replyMsg = addQueueManagerEntry(newQmName);
		reply = handleMessage(replyMsg);
	}
	return reply;
}
/**
 * performOperation method comment.
 */
protected boolean _reallyDeleteQmEntry(String removeQmName) {
	boolean reply = false;
	if (null != removeQmName) {
		UIObject uio = findExistingChild(QUEUEMANAGER_ENTRY_TYPE_NAME, removeQmName);
		if (null != uio) {
			MQeStoreAndForwardQueueAdminMsg replyMsg = removeQueueManagerEntry(removeQmName);
			reply = handleMessage(replyMsg);
		}
	}
	return reply;
}
private final boolean addQueueManager(Renderer renderer, String[] data) {
	boolean reply = false;
	String newQmName = data[0];
	if (null != newQmName) {
		if (null == findExistingChild(QUEUEMANAGER_ENTRY_TYPE_NAME, newQmName)) {
			MQeObject mqeo = (MQeObject)addNewChild(QUEUEMANAGER_ENTRY_TYPE_NAME, newQmName);
			mqeo.commit(false);
		}
	}
	return true;
}
public final MQeStoreAndForwardQueueAdminMsg addQueueManagerEntry(String addQueueManagerName) {
	MQeAdminMsg replyMsg = null;
	MQeStoreAndForwardQueueAdminMsg msg = storeAndForwardQueueAdminMessage();
	try {
		msg.addQueueManager(addQueueManagerName);
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		//_log().println("Error creating queue manager entry");
		//e.printStackTrace(_log());
	}
	return (MQeStoreAndForwardQueueAdminMsg)replyMsg;
}
/**
 * This is the method called to handle all reply messages, whether delivered
 * asynchronously (through the callback method 'acceptMessage') or synchronously
 * as a returned value from a stub operation.
 */
protected boolean handleMessage(MQeAdminMsg msg) {
	boolean changed = false;
	if (null != msg) {
		switch (msg.getAction()) {
			case MQeStoreAndForwardQueueAdminMsg.Action_AddQueueManager :
				changed = _checkAddQmReply((MQeStoreAndForwardQueueAdminMsg) msg);
				break;
			case MQeStoreAndForwardQueueAdminMsg.Action_RemoveQueueManager :
				changed = _checkRemoveQmReply((MQeStoreAndForwardQueueAdminMsg) msg);
				break;
			default :
				changed = super.handleMessage(msg);
				break;
		}
	}
	return changed;
}
/**
 * Perform the actions represented by the supplied instance of 
 * <code>RenderableOperation</code> defined in te metamodel file.
 * If the operation was defined with a prompt then the String array
 * will be non-null, and will contain a string for every line of the prompt
 *
 * The renderer responsible for delivering the operation request is passed in
 * for convenience.
 *
 * Reply true if a stuctural refresh is required.
 */
public boolean performOperation(RenderableOperation op, Renderer renderer) {
	String[] data = op.getData();
	String command = op.getName();
	boolean reply = false;
	if (CMD_ADD_QM.equals(command)) {
		reply = addQueueManager(renderer, data);
	} else {
		reply = super.performOperation(op, renderer);
	}
	return reply;
}
/**
 * Perform the placing of a tespt message.
 */
public boolean performTestMessage(Renderer renderer) {
	String[] data = renderer.promptUserForStrings("Enter TestMessage Data for " + getName(), 
		new String[] {"Queue Name", "text"}, 
		new String[] {"SYSTEM.DEFAULT.LOCAL.QUEUE", "Test Data"});
	if (null != data && data.length > 1) {
		// dont use the super class implementation of testMessage, as it creates a route
		// bypassing the SAFQ
		testMessage(getQueueQueueManagerName(), data[0], data[1]);
	}
	return false;
}
/**
 * reply an admin message for the MQe resource 
 * represented by the receiver.
 */
public MQeQueueAdminMsg queueAdminMessage() {
	return storeAndForwardQueueAdminMessage();
}
public final MQeStoreAndForwardQueueAdminMsg removeQueueManagerEntry(String addQueueManagerName) {
	MQeAdminMsg replyMsg = null;
	MQeStoreAndForwardQueueAdminMsg msg = storeAndForwardQueueAdminMessage();
	try {
		msg.removeQueueManager(addQueueManagerName);
		replyMsg = putAdminMessage(msg);
	} catch (Exception e) {
		//_log().println("Error creating queue manager entry");
		//e.printStackTrace(_log());
	}
	return (MQeStoreAndForwardQueueAdminMsg)replyMsg;
}
/**
 * queueAdminMessage method comment.
 */
public MQeStoreAndForwardQueueAdminMsg storeAndForwardQueueAdminMessage() {
	MQeStoreAndForwardQueueAdminMsg msg = null;
	try {
		msg = new MQeStoreAndForwardQueueAdminMsg();
		msg.setName(getQueueQueueManagerName(), getName());	// name of resource
	} catch (Exception e) {
		// error creating queueAdminMessage
		//e.printStackTrace(_log());
	}
	return msg;
}
public final void testMessage(String queueQueueManagerName, String queueName, String txt) {
	// need to create and delete temporary remote queue def for
	// qqmName, qname

	boolean[] createdConnection = getQueueManager().addTemporaryRoute(queueQueueManagerName);
	if (createdConnection[0]) {
		//boolean[] createdQueue = getQueueManager().createTemporaryRemoteQueue(queueQueueManagerName, queueName);
		//if (createdQueue[0]) {
			getQueueManager().testMessage(queueQueueManagerName, queueName, txt);
		//}
		//if (createdQueue[1]) {
		//	getQueueManager().deleteTemporaryRemoteQueue(queueQueueManagerName, queueName);
		//}
	}
	if (createdConnection[1]) {
		getQueueManager().removeTemporaryRoute(queueQueueManagerName);
	}
}
}
