package com.ibm.uia.metamodel;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.renderingInterface.*;
import java.util.*;
import com.ibm.uia.xml.*;
/**
 * This type represents the metamodel.  It contains:
 *		zero or more attributeTypes
 *		zero or more metaObjects
 */
public class MetaModel extends MetaModelComponent {
	protected static final String METAMODEL_TAG = "metamodel";
	private static final String CLASSNAME = "className";
	private static final String ROOT_TYPE = "rootType";
	private static final String ROOT_NAME = "rootName";
	private String rootType;
	private String rootName;
	private String className;
	/**
	 * A hashtable to map name of attribute type to attribute type
	 * for all attribute types contained within the metamodel.
	 * Only strings should be used as keys.  Only instances of 
	 * AttributeType or its subclasses should be used as values.
	 * The key should be the name of the AttributeType.  If these 
	 * rules are not followed it will all go horribly wrong.
	 * So there!
	 */
	protected Hashtable allAttributeTypes;
	
	/**
	 * A hashtable to map name of metaObject to metaObject
	 * for all metaObjects contained within the metamodel.
	 * Only strings should be used as keys.  Only instances of 
	 * MetaObject or its subclasses should be used as values.
	 * The key should be the name of the metaObject.  If these 
	 * rules are not followed it will all go horribly wrong.
	 * So there!
	 */
	protected Hashtable allMetaObjects;

	/**
	 * A hashtable of all known objects
	 */
	private Hashtable allObjects;

	private Renderer renderer;
/**
 * MetaModel constructor comment.
 */
public MetaModel() {
	super();
}
void _register(MetaObject metaObject) {
	allMetaObjects.put(metaObject.getName(), metaObject);
}
void _register(UIObject obj) {
	allObjects.put(obj.getId(), obj);
}
public static final MetaModel From(Element element) {
	MetaModel mm = null;
	String className = element.getAttribute(CLASSNAME);
	if (null != className) {
		try {
			mm = (MetaModel)InstanceFromName(className);
		} catch (ClassCastException e) {
			System.out.println("ERROR: Class " + className + " is not a subclass of MetaModel");
		}
	}
	if (null == mm) {
		mm = new MetaModel();
	}
	if (null != mm) {
		mm.parseDom(element);
		mm.className = className;
		mm.rootType = element.getAttribute(ROOT_TYPE);
		mm.rootName = element.getAttribute(ROOT_NAME);
	}
	return mm;
}
public static final MetaModel FromFile(String filename) {
	
	Document doc = Parser.ParseFile(filename);
	
	
	if (null != doc) {
		Element elem = doc.getDocumentElement();
		if (null != elem) {
			MetaModel metaModel = MetaModel.From(elem);
			if (null == metaModel) {
				//error("Could not created Metamodel from File: " + filename);
			}
			return metaModel;
		} else {
			//error("File: " + filename + " is not a valid metamodel file (no document element)");
		}
	} else {
		//error("File: " + filename + " is not a valid metamodel file");
	}
	return null;
}
/**
 * create a DOM representation of the receiver using
 * the supplied document.  The Element generated should
 * be returned to the caller, and not appended to the 
 * document.  Subclasses can override this method but
 * are strongly recommended to call the superclass
 * method FIRST and decorate the returned element with their
 * specific data.
 * Note that the superclass is responsible for navigating
 * the standard tree structures, so this responsibility
 * is removed from subclass implementors
 */
protected Element generateDom(Document doc) {
	Element el = super.generateDom(doc);
	el.setAttribute(CLASSNAME, className);
	el.setAttribute(ROOT_TYPE, rootType);
	el.setAttribute(ROOT_NAME, rootName);
	
	Enumeration enum = allAttributeTypes.elements();
	while (enum.hasMoreElements()) {
		AttributeType de = (AttributeType)enum.nextElement();
		Element child = de.generateDom(doc);
		el.appendChild(child);
	}
	enum = allMetaObjects.elements();
	while (enum.hasMoreElements()) {
		MetaObject de = (MetaObject)enum.nextElement();
		Element child = de.generateDom(doc);
		el.appendChild(child);
	}
	return el;
}
public Enumeration getAllMetaObjects() {
	return allMetaObjects.elements();
}
public Enumeration getAllObjects() {
	return allObjects.elements();
}
public Enumeration getAllTypes() {
	return allAttributeTypes.elements();
}
public final MetaModel getMetaModel() {
	return this;
}
public final MetaObject getMetaObject(String name) {
	return (MetaObject)allMetaObjects.get(name);
}
public final String getName() {
	return METAMODEL_TAG;
}
public final UIObject getObject(String id) {
	return (UIObject)allObjects.get(id);
}
/**
 * Reply the string used to identify an XML tag as representing
 * an instance of the receivers class
 */
public final String getTagName() {
	return METAMODEL_TAG;
}
public final AttributeType getType(String name) {
	return (AttributeType)allAttributeTypes.get(name);
}
public static final Object InstanceFromName(String className) {
	Object obj = null;
	if (null != className) {
		try {
			Class objectClass = Class.forName(className);
			obj = objectClass.newInstance();
		} catch (ClassNotFoundException e) {
			System.out.println("ERROR: Class " + className + " not found");
		} catch (IllegalAccessException e) {
			System.out.println("ERROR: Constructor for " + className + " cannot be invoked");
		} catch (InstantiationException e) {
			System.out.println("ERROR: invoking constructor for " + className);
		}
	}
	return obj;
}
/**
 * Create a new attribute type from the given element.
 */
private AttributeType newAttributeType(Element element) {
	AttributeType attType = null;
	if (element.getNodeName().equals(AttributeType.ATTRIBUTE_TYPE)) {
		String mode = element.getAttribute(AttributeType.ATTRIBUTE_TYPE_MODE);
		if (AttributeType.ENUMERATION_MODE.equals(mode)) {
			attType = newEnumeratedAttributeType();
		} else if (AttributeType.INTEGER_MODE.equals(mode)) {
			attType = newIntegerAttributeType();
		} else if (AttributeType.STRING_MODE.equals(mode)) {
			attType = newStringAttributeType();
		}
	}
	if (null != attType) {
		attType.initialiseFrom(element);
		allAttributeTypes.put(attType.getName(), attType);
	}
	return attType;
}
/**
 * Create a new attribute type for the receiver.
 * Subclasses of MetaModel can overide this to create
 * instances of their own subclasses of AttributeType
 */
protected AttributeType newEnumeratedAttributeType() {
	return new EnumeratedType(this);
}
/**
 * Create a new attribute type for the receiver.
 * Subclasses of MetaModel can overide this to create
 * instances of their own subclasses of AttributeType
 */
protected AttributeType newIntegerAttributeType() {
	return new IntegerType(this);
}
/**
 * Create a new metaObject for the recever.
 * Subclasses of MetaModel can overide this to create
 * instances of their own subclasses of MetaObject
 */
protected MetaObject newMetaObject() {
	return new MetaObject(this);
}
public final RenderableObject newObject(String type, String name) {
	RenderableObject ro = null;
	MetaObject mo = getMetaObject(type);
	if (null != mo) {
		ro = mo.createNewObject(name);
	}
	return ro;
}
public RenderableObject newRootObject() {
	return newObject(rootType, rootName);
}
/**
 * Create a new attribute type for the receiver.
 * Subclasses of MetaModel can overide this to create
 * instances of their own subclasses of AttributeType
 */
protected AttributeType newStringAttributeType() {
	return new StringType(this);
}
/**
 * Initialise a child of the receiver from the given Element.
 */ 
protected UIAComponent parseChildElement(Element element) {
	String elemName = element.getNodeName();
	if (AttributeType.ATTRIBUTE_TYPE.equals(elemName)) {
		AttributeType attType = newAttributeType(element);
	} else if (MetaObject.METAOBJECT_TAG.equals(elemName)) {
		MetaObject metaObject = newMetaObject();
		metaObject.initialiseFrom(element);
	} else {
		return super.parseChildElement(element);
	}
	return null;
}
/**
 * Initialise the receiver from the given Element.
 */ 
protected void parseDom(Element element) {
	allAttributeTypes = new Hashtable();
	allMetaObjects = new Hashtable();
	allObjects = new Hashtable();
	super.parseDom(element);
}
/**
 * performOperation method comment.
 */
public boolean performOperation(RenderableOperation op, Renderer renderer) {
	return false;
}
public void setRenderer(Renderer r) {
	renderer = r;
}
}
