package com.ibm.uia.renderer;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import com.ibm.uia.metamodel.*;
import com.ibm.uia.renderingInterface.*;
import com.ibm.uia.xml.*;
import java.awt.*;
import java.awt.event.*;
import com.ibm.uia.renderer.*;
import java.util.*;

/**
 * This type was created in VisualAge.
 */
public class RendererImplementation extends Frame 
	implements WindowListener, ActionListener, Renderer, Reporter {
	private static final String QUIT = "Exit";
	private static final String LOAD_METAMODEL = "Load MetaModel";
	private static final String SHOW_COMMENTS = "Print Comments";
	
	private Vector models;
	private RendererPanel rendererPanel;
	private Menu currentSelectedMenu = null;
	private MenuBar menuBar;
	private String fileFilter;
	private boolean useSelectedMenu = true;
	private boolean showComments = true;
/**
 * Test constructor comment.
 */
public RendererImplementation() {
	super("MetaModel Renderer");
	addWindowListener(this);
	models = new Vector();
	setMenuBar(_createMenuBar(this));
	rendererPanel = new RendererPanel();
	add(rendererPanel);
	rendererPanel.setRenderer(this);
}
public final void _comment(String s) {
	if (showComments) {
		System.out.println("Comment: " + s);
	}
}
public final void _comment(String s, Exception e) {
	if (showComments) {
		System.out.println("Comment: " + s + e.getMessage());
		e.printStackTrace();
	}
}
private final MenuBar _createMenuBar(ActionListener actList) {
	menuBar = new MenuBar();

	Menu menu = new Menu("File");
	menuBar.add(menu);
	MenuItem menuItem;
	
	//menuItem = new MenuItem(LOAD_METAMODEL);
	//menuItem.addActionListener(actList);
	//menu.add(menuItem);
	
	menuItem = new MenuItem(SHOW_COMMENTS);
	menuItem.addActionListener(actList);
	menu.add(menuItem);
	menuItem = new MenuItem(QUIT);
	menuItem.addActionListener(actList);
	menu.add(menuItem);
	return menuBar;
}
private final Menu _createMetamodelMenu(RenderableObject robj) {
	Enumeration enum = robj.getAllOperations();
	OperationsMenu menu = null;
	if (enum.hasMoreElements()) {
		menu = new OperationsMenu(robj.getCaptionText());
		while (enum.hasMoreElements()) {
			RenderableOperation rop = (RenderableOperation)enum.nextElement();
			menu.add(rop, this);
		}
	}
	return menu;
}
public final void _error(String errorText) {
	System.out.println("Error: " + errorText);
	displayMessage("Error", errorText);
}
public final void _error(String s, Exception e) {
	System.out.println("Error: " + s + e.getMessage());
	e.printStackTrace();
	displayMessage("Error", s + e.getMessage());
}
private final void _exit(boolean cleanly) {
	if (cleanly) {
	}
	System.exit(0);
}
public final void _warning(String warningText) {
	System.out.println("Warning: " + warningText);
	displayMessage("Warning", warningText);
}
public final void _warning(String s, Exception e) {
	System.out.println("Warning: " + s + e.getMessage());
	e.printStackTrace();
	displayMessage("Warning", s + e.getMessage());
}
public boolean accept(java.io.File file, String filename) {
	return filename.endsWith(fileFilter);		
}
public boolean accept(String dir, String filename) {
	return filename.endsWith("xml");		
}
public void actionPerformed(ActionEvent e) {
	String cmd = e.getActionCommand();
	if (QUIT.equals(cmd)) {
		_exit(false);
	} else if (LOAD_METAMODEL.equals(cmd)) {
		String mmfile = getLoadFileName("Select Metamodel File", null, "*.xml");
		if (null != mmfile) {
			addMetaModel(mmfile);
		}
	} else if (SHOW_COMMENTS.equals(cmd)) {
		showComments = ! showComments;
	} else {
		Object obj = e.getSource();
		if (obj instanceof OperationsMenuItem) {
			OperationsMenuItem omi = (OperationsMenuItem) obj;
			RenderableOperation rop = omi.getRenderableOperation();
			if (rop.perform(this)) {
				rendererPanel.display(true);
			}
		}
	}
}
public final void addMetaModel(MetaModel metaModel) {
	addModel(metaModel.newRootObject());
}
public final void addMetaModel(String filename) {
	Document doc = Parser.ParseFile(filename);
	if (null != doc) {
		Element elem = doc.getDocumentElement();
		if (null != elem) {
			MetaModel metaModel = MetaModel.From(elem);
			if (null != metaModel) {
				addMetaModel(metaModel);
			} else {
				error("Could not created Metamodel from File: " + filename);
			}
		} else {
			error("File: " + filename + " is not a valid metamodel file (no document element)");
		}
	} else {
		error("File: " + filename + " is not a valid metamodel file");
	}
}
public final void addModel(RenderableObject model) {
	models.addElement(model);
	menuBar.add(_createMetamodelMenu(model));
	setSelectedMenu(currentSelectedMenu);	// remove and add should move it to end
	model.setRenderer(this);
	redisplay(true);
}
public final void comment(String commentText) {
	if (showComments) {
		System.out.println("Comment: " + commentText);
	}
}
public final void displayMessage(String title, String message) {
	MessageDialogue dlg = new MessageDialogue(this, title, message);
	dlg.show();
}
/**
 * edit the given object if possible.
 * return true if a change was made.
 */
public final boolean edit(RenderableObject rend, boolean displayOnlyRequiredAttributes) {
	UIOEditDialogue dlg = new UIOEditDialogue(this, "Edit Dialog", rend);
	dlg.show();
	//redisplay(false);
	return dlg.okPressed;
}
public final void error(String errorText) {
	System.out.println("Error: " + errorText);
	displayMessage("Error", errorText);
}
/**
 * Get a load file description from the user.
 * 	title:  dialog title
 *	dir:	name of initial directory  (current directory if null)
 *	dir:	initial name of file	(ignored if null)
 *
 * returns a complete file description
 */
public final String getLoadFileName(String title, String dir, String filename) {
	FileDialog dlg = new FileDialog(this, title, FileDialog.LOAD);
	if (null != dir) {
		dlg.setDirectory(dir);
	} else {
		dlg.setDirectory(new java.io.File("").getAbsolutePath());
	}
	if (null != filename) {
		dlg.setFile(filename);
	}
	dlg.show();
	String reply = null;
	dir = dlg.getDirectory();
	filename = dlg.getFile(); 
	if (null != dir && null != filename) {
		reply = dir + filename;
	}
	return  reply; 
}
public final Enumeration getRootObjects() {
	return models.elements();
}
/**
 * Get a save file description from the user.
 * 	title:  dialog title
 *	dir:	name of initial directory  (current directory if null)
 *	filename:	initial name of file	(ignored if null)
 *
 * returns a complete file description
 */
public final String getSaveFileName(String title, String dir, String filename) {
	FileDialog dlg = new FileDialog(this, title, FileDialog.SAVE);
	if (null != dir) {
		dlg.setDirectory(dir);
	} else {
		dlg.setDirectory(new java.io.File("").getAbsolutePath());
	}
	if (null != filename) {
		dlg.setFile(filename);
	}
	dlg.show();
	String reply = null;
	dir = dlg.getDirectory();
	filename = dlg.getFile(); 
	if (null != dir && null != filename) {
		reply = dir + filename;
	}
	return  reply; 
}
public static void main(java.lang.String[] args) {
	RendererImplementation test = new RendererImplementation();
	for (int i = 0; i < args.length; i++){
		if ("-smenu".equalsIgnoreCase(args[i])) {
			test.useSelectedMenu = false;
		} else if ("-pmenu".equalsIgnoreCase(args[i])) {
			test.usePopupMenu(false);
		} else {
			test.addMetaModel(args[i]);
		}
	}
	test.setBounds(0,0,610, 450);
	test.show();
}
public void openNewRenderer(RenderableObject robj) {
	RendererImplementation test = new RendererImplementation();
	if (null != robj) {
		test.addModel(robj);
	}
	test.setBounds(0,0,610, 450);
	test.show();
}
public void paint(Graphics g) {
	//paintComponents(g);
}
public String[] promptUserForStrings(String title, String[] fields, String[] defaults) {
	String[] reply = null;
	GetStringsDialogue dlg = new GetStringsDialogue(this, title, fields, defaults);
	dlg.show();
	if (dlg.okPressed()) {
		reply = dlg.getData();
	}
	return reply;
}
public final void redisplay(boolean recalc) {
	rendererPanel.display(recalc);	
}
public final void removeModel(RenderableObject model) {
	models.removeElement(model);
}
public final void setSelectedMenu(java.awt.Menu menu) {
	
	MenuBar mb = getMenuBar();
	if (null != currentSelectedMenu) {
		mb.remove(currentSelectedMenu);
	}
	if (useSelectedMenu) {
		currentSelectedMenu = menu;
		if (null != currentSelectedMenu) {
			mb.add(currentSelectedMenu);
		}
	}
}
public final void usePopupMenu(boolean use) {
	rendererPanel.usePopupMenu(use);
}
public final void useSelectedMenu(boolean use) {
	useSelectedMenu = use;
}
public final void warning(String warningText) {
	System.out.println("Warning: " + warningText);
	displayMessage("Warning", warningText);
}
/**
 * windowActivated method comment.
 */
public void windowActivated(java.awt.event.WindowEvent e) {
}
/**
 * windowClosed method comment.
 */
public void windowClosed(java.awt.event.WindowEvent e) {
}
/**
 * windowClosing method comment.
 */
public void windowClosing(java.awt.event.WindowEvent e) {
	_exit(false);
}
/**
 * windowDeactivated method comment.
 */
public void windowDeactivated(java.awt.event.WindowEvent e) {
}
/**
 * windowDeiconified method comment.
 */
public void windowDeiconified(java.awt.event.WindowEvent e) {
}
/**
 * windowIconified method comment.
 */
public void windowIconified(java.awt.event.WindowEvent e) {
}
/**
 * windowOpened method comment.
 */
public void windowOpened(java.awt.event.WindowEvent e) {
}
}
