package com.ibm.uia.xml;

/****************************************************************************/
/*                                                                          */
/* Program name: MQ Series Everyplace - Administration Tool                 */
/*                                                                          */
/*  Statement:  Licensed Materials - Property of IBM                        */
/*                                                                          */
/*              MQSeries SupportPac MSE1                                    */
/*              (C) Copyright IBM Corp. 2000                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/* Change history:                                                          */
/*                                                                          */
/* V1.0   15-08-2000  PTvL  Initial release                                 */
/*                                                                          */
/*==========================================================================*/
import java.io.*;
import java.util.*;

public final class Element implements XMLConstants {
	protected static final String TRUE = "true";
	protected static final String FALSE = "false";
	protected static final String NEWLINE = "\n";
	protected static final String TAB = "    ";

	private int startPos;
	private int endPos;		
		
	private boolean isEmptyTag = false;		// a tag with no children eg: <fred />
	private boolean isEndTag = false;		// a tag that starts with a forward slash </fred>
	private String nodeName;
	private Vector children;
	private Hashtable attributes;
/**
 * ElementImpl constructor comment.
 */
public Element() {
	super();
	attributes = new Hashtable();
	children = new Vector();
}
/**
 * ElementImpl constructor comment.
 */
public Element(int count) {
	this();
	startPos = count;
}
/**
 * ElementImpl constructor comment.
 */
public Element(String nodeName) {
	this();
	this.nodeName = nodeName;
}
private void _generateXML(Writer writer, int pretty) {
	try {
		// indent....
		for (int i = 0; i < pretty; i++) {
			writer.write(TAB);
		}
		writer.write(START_OF_TAG);
		writer.write(getName());
		Enumeration enum = getAttributeNames();
		while (enum.hasMoreElements()) {
			writer.write(SPACE);
			String key = enum.nextElement().toString();
			writer.write(key);
			writer.write(EQUALS);
			writer.write(QUOTE);
			writer.write(getAttribute(key));
			writer.write(QUOTE);
		}
		enum = getChildren();
		if (!enum.hasMoreElements()) {
			writer.write(FORWARD_SLASH);
			writer.write(END_OF_TAG);
		} else {
			writer.write(END_OF_TAG);
			int newPretty = (pretty < 0) ? pretty : pretty + 1;
			while (enum.hasMoreElements()) {
				writer.write(NEWLINE);
				Element tag = (Element)enum.nextElement();
				tag._generateXML(writer, newPretty);
			}

			// write an end tag
			writer.write(NEWLINE);
			for (int i = 0; i < pretty; i++) {
				writer.write(TAB);
			}
			writer.write(START_OF_TAG);
			writer.write(FORWARD_SLASH);
			writer.write(getName());
			writer.write(END_OF_TAG);
		}
		writer.flush();
	} catch (IOException e) {
		System.out.println(e);
		e.printStackTrace();
	}	
}
final boolean _isEmptyTag() {
	return isEmptyTag;
}
final boolean _isEndTag() {
	return isEndTag;
}
final boolean _isEndTagFor(Element tag) {
	return getName().equals(tag.getName());
}
void _setEndPos(int end) {
	endPos = end;
}
final void _setIsEmptyTag(boolean set) {
	isEmptyTag = set;
}
final void _setIsEndTag(boolean set) {
	isEndTag = set;
}
final void _setName(String str) {
	setNodeName(str);
}
public void addChild(Element tag) {
	appendChild(tag);
	isEmptyTag = false;
}
/**
 * appendChild method comment.
 */
public Element appendChild(Element newChild) {
	children.addElement(newChild);
	return newChild;
}
public void generateXML(Writer writer, boolean pretty) {
	int i = pretty ? 0 : -1;
	_generateXML(writer, i);
}
/**
 * getAttribute method comment.
 */
public String getAttribute(String name) {
	return (String)attributes.get(name);
}
public final boolean getAttributeBoolean(String attrName) {
	boolean bVal = false;
	String sVal = getAttributeValue(attrName);
	if (null != sVal) {
		bVal = sVal.equals(TRUE);
	}
	return bVal;
}
public final int getAttributeInteger(String attrName) {
	int iVal = 0;
	String sVal = getAttributeValue(attrName);
	if (null != sVal) {
		iVal = Integer.parseInt(sVal);
	}
	return iVal;
}
/**
 * getAttribute method comment.
 */
public final Enumeration getAttributeNames() {
	return attributes.keys();
}
public final String getAttributeValue(String attrName) {
	return getAttribute(attrName);
}
/**
 * getChildNodes method comment.
 */
public Enumeration getChildren() {
	return children.elements();
}
public final String getName() {
	return getNodeName();
}
/**
 * getNodeName method comment.
 */
public String getNodeName() {
	return nodeName;
}
public final void prettyPrint(int indent) {
	for (int i = 0; i < indent; i++) {
		System.out.print("\t");
	}
	System.out.println(toString());
	Enumeration enum = getChildren();
	while (enum.hasMoreElements()) {
		((Element)enum.nextElement()).prettyPrint(indent + 1);
	}
}
/**
 * removeAttribute method comment.
 */
public void removeAttribute(String name) {
	//attributes.remove(name);
}
public final void setAttribute(String attrName, int value) {
	setAttribute(attrName, Integer.toString(value));
}
/**
 * setAttribute method comment.
 */
public void setAttribute(String name, String value) {
	if (null != name && null != value) {
		attributes.put(name, value);
	}
}
public final void setAttribute(String attrName, boolean value) {
	if (value) {
		setAttribute(attrName, TRUE);
	} else {
		setAttribute(attrName, FALSE);
	}
}
/**
 * setNodeValue method comment.
 */
public void setNodeName(String name) {
	nodeName = name;
}
public String toString() {
	return Unparser.UnparseToString(this, true);
}
}
